% CalcCheck.lhs
%
% This file is part of CalcCheck
%
% Copyright (c) 2011  Wolfram Kahl
% All rights reserved
%
\section{|CalcCheck| Program}

\begin{code}
module Main (main) where

import Version
import Options (CalcCheckOpts(..), getOptsAndArgs, programId)
import Parser (pFile)
import LogicE (showsItems, showsItemsHTML, itemsSummary')
import CheckCalcs (checkItems)

import Text.ParserCombinators.Parsec (parseFromFile)

import System.IO (hSetEncoding, hFlush, stdout, utf8)
import qualified System.FilePath as FilePath
import System.Directory (getCurrentDirectory)
import System.Mem (performGC)
\end{code}



\begin{code}
main :: IO ()
main = do
    hSetEncoding stdout utf8
    (cco, args) <- getOptsAndArgs
    main' cco args

main' :: CalcCheckOpts -> [String] -> IO ()
main' cco (a : _) = run cco a
main' cco [] = do
  wd <- getCurrentDirectory
  putStrLn $ version ++ " running in " ++ wd
  putStr "Argument file name (press return to exit): "
  hFlush stdout
  s <- getLine
  if null s
    then return ()
    else if FilePath.isValid s
      then checkInteractive cco s
      else do
        putStrLn "Not a valid file path!"
        main' cco []

checkInteractive :: CalcCheckOpts -> String -> IO ()
checkInteractive cco file = do
        run cco file
        putStrLn $ version ++ " done."
        putStr $ "Do you want to re-check `" ++ file ++ "'? [Ny] "
        hFlush stdout
        performGC
        s <- getLine
        if take 1 s `elem` ["Y" , "y"]
          then checkInteractive cco file
          else return ()

run :: CalcCheckOpts -> String -> IO ()
run cco file = do
      ecss <- parseFromFile pFile file
      case ecss of
        Left e -> putStrLn $ programId cco ++ ": syntax error: " ++ show e
        Right css -> let
              checked = checkItems css
              htmlFile = file ++ ".html"
           in do  putStrLn  $ programId cco ++ ": " ++ file ++ ": No syntax errors."
                  putStrLn  $ programId cco ++ ": Now checking...\n"
                  if not $ opt_quiet cco
                    then putStrLn $ showsItems cco checked ""
                    else return ()
                  writeFile htmlFile $ wrapHtml file $
                      showsItemsHTML cco checked ""
                  putStrLn  ""
                  putStrLn  $ programId cco ++ ": Finished checking " ++ file
                  putStr  $ itemsSummary' cco checked
                  putStrLn  $ programId cco ++ ": Wrote " ++ htmlFile
\end{code}


\begin{code}
wrapHtml :: FilePath -> String -> String
wrapHtml name s = let title = version ++ " output for " ++ name
  in unlines
  ["<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2//EN\">"
  ,"<html>"
  ,"<head><title>" ++ title ++ "</title>"
  ,"<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"/>"
  ,"</head>"
  ,"<body style=\"line-height:130%\" bgcolor=\"#FFFFFF\">"
--  ,"<h1 align=\"center\">" ++ title ++ "</h1>"
  ,s
  ,"</body>"
  ,"</html>"
  ]
\end{code}


%{{{ EMACS lv
% Local Variables:
% folded-file: t
% fold-internal-margins: 0
% eval: (fold-set-marks "%{{{ " "%}}}")
% eval: (fold-whole-buffer)
% end:
%}}}
