% Utils/Q.lhs
%
% This file is part of CalcCheck
%
% Copyright (c) 2011  Wolfram Kahl
% All rights reserved
%
\section{|Utils.Q|: Q-Combinators}

Here is a Haskell version of the Q-combinators
used by John Harrison's HOL-light.

\begin{code}
module Utils.Q where
\end{code}

%{{{ qDo, qList
\begin{code}
qDo :: (a -> Maybe a) -> a -> a
qDo f x = case f x of Nothing -> x; Just y -> y

-- The argument of |qDoMaybe| returns |Just Nothing| for no change:
qDoMaybe :: (a -> Maybe (Maybe a)) -> a -> Maybe a
qDoMaybe f x = case f x of
   Nothing -> Nothing
   Just m -> Just $ case m of
     Nothing -> x
     Just y -> y

qPair :: (a -> Maybe a) -> (b -> Maybe b) -> (a, b) -> Maybe (a , b)
qPair f g (x, y) = case f x of
  Nothing -> fmap ((,) x) $ g y
  Just x' -> Just $ (x' , qDo g y)

q2 :: (a -> b -> c) -> (a -> Maybe a) -> (b -> Maybe b) -> a -> b -> Maybe c
q2 op f g x y = fmap (uncurry op) (qPair f g (x, y))

qList :: (a -> Maybe a) -> [a] -> Maybe [a]
qList _ [] = Nothing
qList f (x : xs) = case f x of
  Nothing -> fmap (x :) $ qList f xs
  Just y -> Just $ y : qDo (qList f) xs

qListMaybe :: (a -> Maybe (Maybe a)) -> [a] -> Maybe (Maybe [a])
qListMaybe _ [] = Just Nothing
qListMaybe f (x : xs) = case f x of
  Nothing -> Nothing
  Just m -> case m of
    Nothing -> fmap (fmap (x :)) $ qListMaybe f xs
    Just y -> fmap (Just . (y :)) $ qDoMaybe (qListMaybe f) xs
\end{code}
%}}}

%{{{ EMACS lv
% Local Variables:
% folded-file: t
% fold-internal-margins: 0
% eval: (fold-set-marks "%{{{ " "%}}}")
% eval: (fold-whole-buffer)
% end:
%}}}
